
#include <avr/io.h>

#ifndef F_CPU
#define F_CPU 16000000
#endif

#include <util/delay.h>
#include <string.h>
#include "lib_lcd.h"


void lcd_enable( void )
{
    LCD_CONF |= (1 << LCD_EN); 
    _delay_us( LCD_ENABLE_US );
    LCD_CONF &= ~(1<<LCD_EN); 
}

void lcd_out( uint8_t data )
{
    LCD_DATA = data & 0b11110000;
    lcd_enable();

	LCD_DATA = (data & 0b00001111) << 4;
    lcd_enable();
}


void lcd_command( uint8_t data )
{
    LCD_CONF &= ~(1<<LCD_RS);    // RS auf 0 setzen 
    lcd_out( data );             // zuerst die oberen, 
    _delay_us( LCD_COMMAND_US );
}

void lcd_clear( void )
{
    lcd_command( LCD_CLEAR_DISPLAY );
    _delay_ms( LCD_CLEAR_DISPLAY_MS );
}


void lcd_init( void )
{
    
    LCD_DATA_DR = 0b11110000;
	LCD_CONF_DR = (1<<LCD_EN) | (1<<LCD_RW) | (1<<LCD_RS);
 	


    LCD_DATA = 0x00;

    _delay_ms( LCD_BOOTUP_MS );

	LCD_DATA = 0x00;

	lcd_enable();

	lcd_out( LCD_SOFT_RESET );
    _delay_ms( LCD_SOFT_RESET_MS1 );
 
    lcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS2 );
 
    lcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS3 );


    lcd_out( LCD_SET_FUNCTION |
             LCD_FUNCTION_8BIT );

    _delay_ms( LCD_SET_4BITMODE_MS );
 
    // 4-bit Modus / 2 Zeilen / 5x7
    lcd_command( LCD_SET_FUNCTION |
                 LCD_FUNCTION_4BIT |
                 LCD_FUNCTION_2LINE |
                 LCD_FUNCTION_5X7 );
 
    // Display ein / Cursor aus / Blinken aus
    lcd_command( LCD_SET_DISPLAY |
                 LCD_DISPLAY_ON |
                 LCD_CURSOR_OFF |
                 LCD_BLINKING_OFF); 
 
    // Cursor inkrement / kein Scrollen
    lcd_command( LCD_SET_ENTRY |
                 LCD_ENTRY_INCREASE |
                 LCD_ENTRY_NOSHIFT );

	lcd_clear();


}


void lcd_data( uint8_t data )
{
    LCD_CONF |= (1<<LCD_RS);    // RS auf 1 setzen
    lcd_out( data );            // zuerst die oberen, 
    _delay_us( LCD_WRITEDATA_US );
}


void lcd_string( const char *data ) {
    while( *data != '\0' )
        lcd_data( *data++ );
}

void lcd_home( void ) {
    lcd_command( LCD_CURSOR_HOME );
    _delay_ms( LCD_CURSOR_HOME_MS );
}

void lcd_setcursor( uint8_t x, uint8_t y ) {
    uint8_t data;
 
    switch (y) {
        case 0:    // 1. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE1 + x;
            break;
        case 1:    // 2. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE2 + x;
            break;
        case 2:    // 3. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE3 + x;
            break;
        case 3:    // 4. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE4 + x;
            break;
        default:    
            return; // fr den Fall einer falschen Zeile
    }
    lcd_command( data );
}

void lcd_string_xy( uint8_t x, uint8_t y, const char *data ) {
    lcd_setcursor( x, y );
    lcd_string( data );
}

